﻿-- Author      : grunkalunca
-- Create Date : 2/5/2010 10:51:36 PM

local TradeShareMulti = CreateFrame("Frame","TradeShareMulti")
TradeShareMulti:RegisterEvent("ADDON_LOADED");

local playerGuild = ""
local realm = GetRealmName()
local guildDict = {}
local frameMultiOpened = false
local chatEvents = { 
	["CHAT_MSG_CHANNEL"] = true,
	["CHAT_MSG_GUILD"] = true,
	["CHAT_MSG_OFFICER"] = true,
	["CHAT_MSG_PARTY"] = true,
	["CHAT_MSG_RAID"] = true,
	["CHAT_MSG_SYSTEM"] = true,
	["CHAT_MSG_SAY"] = true,
	["CHAT_MSG_TEXT_EMOTE"] = true,
	["CHAT_MSG_WHISPER"] = true,
	["CHAT_MSG_YELL"] = true,
}

function getTitleString(linkCount)
	return linkCount .. " professions ready to post"
end

function dictLen(dict)
	count = 0
	for _,_ in pairs(dict) do
		count = count + 1
	end

	return count
end

function joinDictKeys(dict, delim)
	text = ""
	for k,v in pairs(dict) do
		if text == "" then
			text = k
		else
			text = text .. delim .. k
		end
	end

	return text
end

function joinDictValues(dict, delim)
	text = ""
	for k,v in pairs(dict) do
		if text == "" then
			text = v[2]
		else
			text = text .. delim .. v[2]
		end
	end

	return text
end

function getGuild(charName)
	local guild = guildDict[charName], rank, rankId

	if guild ~= nil and guild ~= "None" then
		--only return
		return guild
	elseif charName == UnitName("player") then
		--if the player is you
		guild = getMyGuild()
	else
		guild = "None"
	end

	if charName ~= nil then
		guildDict[charName] = guild
	end

	return guild
end

function TradeShareMulti:OnEvent(event,arg1, arg2, ...)
	if chatEvents[event] then

		local msg = arg1
		
		--assumes players sending links are themselves
		local charName = arg2

		--if can't get guild, will be set to None where we will attempt to lookup server side
		local guild = getGuild(charName)

		if guild == "None" and event == "CHAT_MSG_GUILD" then
			guild = getGuild(UnitName("player")) --assume that if it shows in guild chat, they are in your guild
		end

		--iterate through the links contained in the message
		local foundLinks = false
		local linkTable = FindTradeLinks(msg)
		for _,linkmsg in ipairs(linkTable) do
			local link, guid, profession = ParseTradeLink(linkmsg)

			if link ~= nil then
				if LINK_DICTIONARY[realm] == nil then
					LINK_DICTIONARY[realm] = {}
				end
				LINK_DICTIONARY[realm][charName .. ":" .. profession] = {time(), CreateTradeShareLink(link, charName, guild, realm, "")}
				foundLinks = true
			end
		end	
		
		if foundLinks then
			--update window text
			Frame_Multi_FontStringTitle:SetText(getTitleString(dictLen(LINK_DICTIONARY[realm])))

			--update copy text
			if frameMultiOpened then
				Frame_Multi:Hide()
				Frame_Multi_EditBox:SetText("")			
				Frame_Multi_EditBox:SetText(joinDictValues(LINK_DICTIONARY[realm], "\n"))
			
				Frame_Multi:Show()
				Frame_Multi_EditBox:SetFocus()
			end
		end
	elseif event == "ADDON_LOADED" and arg1 == "TradeShare" then
		--init dictionaries if necessary
		if LINK_DICTIONARY == nil then
			LINK_DICTIONARY = {}
		end

		if LINK_DICTIONARY[realm] == nil then
			LINK_DICTIONARY[realm] = {}
		end

		--remove stale links (older than 3 days)
		for k,v in pairs(LINK_DICTIONARY[realm]) do
			LINK_DICTIONARY[realm][k] = ValidateTradeShareLink(v)
		end

		if dictLen(LINK_DICTIONARY[realm]) > 0 then
			tsPrint("TradeShare: You have " .. dictLen(LINK_DICTIONARY[realm]) .. " links to share, type /tradeshare")
		end
	end
end

local sysChatInfo = ChatTypeInfo["SYSTEM"];
function tsPrint(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg, sysChatInfo.r, sysChatInfo.g, sysChatInfo.b, sysChatInfo.id);
end


SLASH_TRADESHARE1 = '/tradeshare'
SLASH_TRADESHARE2 = '/TradeShare'
SLASH_TRADESHARE3 = '/TRADESHARE'
SLASH_TRADESHARE4 = '/Tradeshare'
function SlashCmdList.TRADESHARE(msg, editbox)
	tsPrint (msg)
	tsPrint ("TradeShare: Post professions on wowtradeshare.com")

	if msg == "list" then
		tsPrint ("Tradelinks collected so far:")
		tsPrint ("-" .. joinDictKeys(LINK_DICTIONARY[realm], "\n-"))
	elseif msg == "clear" or msg == "delete" then
		tsPrint ("All collected tradelinks have been deleted")
		LINK_DICTIONARY[realm] = {}

		--if the window was open, redraw it
		if frameMultiOpened then
			TradeShareMulti:OnClick()
		end
	else
		tsPrint ("  Tradelinks appearing in chat(guild, trade, etc...) are automatically collected")
		tsPrint ("  Share your own tradelinks by clicking the 'share' button on your tradeskill windows")
		tsPrint ("  Tradelinks you collect will be cached for up to 72 hours")
		tsPrint ("")
		tsPrint ("usage:")
		tsPrint ("/tradeshare [ list | clear ]")
		tsPrint ("  default - opens the TradeShare multi-post window")
		tsPrint ("  list - Lists all tradelinks that have been collected")
		tsPrint ("  clear - Clears the list of collected tradelinks")
		TradeShareMulti:OnClick()
	end
end

function TradeShareMulti:Initialize()
	if self.initialized then return end 

	--start listening for guild input
	for event,_ in pairs(chatEvents) do
		self:RegisterEvent(event)
	end

	frameMultiOpened = false
	self.initialized = true

	Frame_Multi:Hide()
end

function TradeShareMulti:OnClick()
	frameMultiOpened = true
	Frame_Single:Hide()
	Frame_Multi:Hide()
	Frame_Multi_FontStringTitle:SetText(getTitleString(dictLen(LINK_DICTIONARY[realm])))
	Frame_Multi_EditBox:SetText("")
	Frame_Multi_EditBox:SetText(joinDictValues(LINK_DICTIONARY[realm], "\n"))
	Frame_Multi:Show()
	Frame_Multi_EditBox:SetFocus()
end

function ButtonOk_OnClick()
	frameMultiOpened = false
	Frame_Multi:Hide()
end

function Frame_Multi_EditBox_OnTextChanged()
	return
end

function Frame_Multi_OnLoad()
	TradeShareMulti:SetScript("OnEvent",TradeShareMulti.OnEvent)
	TradeShareMulti:Initialize()
end


